package featureide_ui.editors.grammar;

import org.eclipse.core.resources.IFile;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.SWTGraphics;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.LayerManager;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.Display;

/**
 * @author Jakub Jurkiewicz
 */
public class ImageSaveUtil {

	/**
	 * Saves the editor screenshot to the graphical file
	 * 
	 * @param file
	 *            URI path of the file to be saved
	 * @param format
	 *            format of graphical file. Allowed values are: SWT.IMAGE_BMP,
	 *            SWT.IMAGE_GIF, SWT.IMAGE_JPEG, SWT.IMAGE_PNG
	 * @param viewer
	 *            edit part viewer of an editor
	 * @param screen
	 *            <code>Screen</code> element
	 * @return <code>true</code> when image file was saved succesfully
	 */
	public static boolean createScreenshot(IFile file, int format,
			EditPartViewer viewer) {
		SWTGraphics graphics = null;
		GC gc = null;
		Image image = null;
		try {
			ScalableFreeformRootEditPart rootEditPart = (ScalableFreeformRootEditPart) viewer
					.getContents().getRoot();
			LayerManager layerManager = (LayerManager) viewer
					.getEditPartRegistry().get(LayerManager.ID);
			IFigure figure = layerManager
					.getLayer(LayerConstants.PRINTABLE_LAYERS);
			Rectangle rectangleBounds = figure.getBounds();

			// viewport which displays my diagram
			Viewport viewport = (Viewport) rootEditPart.getFigure();
			int imageWidthScaled = rectangleBounds.width;
			int imageHeightScaled = rectangleBounds.height;

			// creating new Image
			image = new Image(Display.getDefault(), imageWidthScaled,
					imageHeightScaled);
			gc = new GC(image);
			graphics = new SWTGraphics(gc);
			// paint contents on the Image
			viewport.paint(graphics);
			// saving an image to the disk
			saveImage(file, format, image);

			return true;

		} catch (Exception e) {
			return false;
		} finally {
			// we do not need this elements anymore, so we dispose them
			if (graphics != null) {
				graphics.dispose();
			}
			if (gc != null) {
				gc.dispose();
			}
			if (image != null) {
				image.dispose();
			}
		}
	}

	/**
	 * Saves the image
	 * 
	 * @param file
	 *            <code>URI</code> of the original file
	 * @param format
	 *            format of the image file
	 * @param image
	 */
	private static void saveImage(IFile file, int format, Image image) {
		ImageLoader loader = new ImageLoader();
		loader.data = new ImageData[1];
		loader.data[0] = image.getImageData();
		String filePath = createImageFilePath(file, format);
		loader.save(filePath, format);
	}

	/**
	 * Gets the image file extenson according to the image format
	 * 
	 * @param format
	 *            format of the image file
	 * @return file extension related with image format
	 */
	public static String getImageFileExtension(int format) {
		if (format == SWT.IMAGE_BMP) {
			return "bmp";
		} else if (format == SWT.IMAGE_GIF) {
			return "gif";
		} else if (format == SWT.IMAGE_ICO) {
			return "ico";
		} else if (format == SWT.IMAGE_JPEG) {
			return "jpg";
		} else if (format == SWT.IMAGE_PNG) {
			return "png";
		} else {
			return null;
		}
	}
    
    /**
     * This method works in the opposite way than 
     * {@link ImageSaveUtil#getImageFileExtension(int)}
     * 
     * @param extension
     * @return file format or -1 if unrecognized
     */
    public static int getImageFormat(String extension) {
        if ("bmp".equalsIgnoreCase(extension)) {
            return SWT.IMAGE_BMP;
        } else if ("gif".equalsIgnoreCase(extension)) {
            return SWT.IMAGE_GIF;
        } else if ("ico".equalsIgnoreCase(extension)) {
            return SWT.IMAGE_ICO;
        } else if ("jpg".equalsIgnoreCase(extension)) {
            return SWT.IMAGE_JPEG;
        } else if ("png".equalsIgnoreCase(extension)) {
            return SWT.IMAGE_PNG;
        } else {
            return -1;
        }
    }

	/**
	 * Creates image file path
	 * 
	 * @param file
	 *            path of the original file
	 * @param format
	 *            format of the image file
	 * @return path of the image file
	 */
	public static String createImageFilePath(IFile file, int format) {
		String screenPathName = file.getFullPath().toOSString();
		return changeFileExtensions(screenPathName, format);

	}

	/**
	 * Changes the file's extension
	 * 
	 * @param path
	 *            file's path
	 * @param format
	 *            format of the image file
	 * @return fileName with changed extension
	 */
	private static String changeFileExtensions(String path, int format) {
		int extensionIndex = path.lastIndexOf(".");
		String filePath = path.substring(0, extensionIndex + 1); // we
		// substring
		// with dot
		filePath += getImageFileExtension(format);
		return filePath;
	}
}