package featureide_ui.editors.grammar.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.FeatureUIHelper;

/**
 * This command allows the user to move features at the feature diagram using
 * drag and drop.
 * 
 * @author Thomas Thm
 *
 */
public class FeatureDragAndDropCommand extends Command {

	private final FeatureModel featureModel;

	private final Feature feature;

	private final Point newLocation;
	
	private final Feature oldParent;
	
	private final int oldIndex;
	
	private Feature newParent;
	
	private int newIndex;

	public FeatureDragAndDropCommand(FeatureModel featureModel, Feature feature, Point newLocation) {
		super("Moving " + feature.getName());
		this.featureModel = featureModel;
		this.feature = feature;
		this.newLocation = newLocation;
		oldParent = feature.getParent();
		oldIndex = oldParent != null ? oldParent.getChildIndex(feature) : 0;
	}

	@Override
	public boolean canExecute() {
		Point referencePoint = FeatureUIHelper.getSourceLocation(feature,newLocation);
		Feature next = calculateNext(featureModel.getRoot(), referencePoint);
		
		//calculate new parent (if exists)
		if (!calculateNewParentAndIndex(next))
			return false;
	
		//no new positions possible next to same feature
		if (next == feature) 
			return false;
		
		//not accept the same position
		if (oldParent == newParent && oldIndex == newIndex)
			return false;
		
		//not accept leaves as parent
		if (newParent.isLayer())
			return false;
		
		//not accept moves to children positions
		return feature != newParent && !feature.isAncestorOf(newParent);
	}

	@Override
	public void execute() {
		oldParent.removeChild(feature);
		newParent.addChildAtPosition(newIndex, feature);
		if (oldParent.isLayer())
			featureModel.deleteFeature(oldParent);
		featureModel.handleModelDataChanged();
	}

	@Override
	public void undo() {
		newParent.removeChild(feature);
		//TODO implement undo for deleted compound features 
		if (oldParent != null) 
			oldParent.addChildAtPosition(oldIndex, feature);
		featureModel.handleModelDataChanged();
	}
	
	private boolean calculateNewParentAndIndex(Feature next) {
		Point location = FeatureUIHelper.getSourceLocation(feature,newLocation);
		Point nextLocation = FeatureUIHelper.getTargetLocation(next);
		Dimension d = location.getDifference(nextLocation);
		
//		if (Math.abs(d.width) < d.height) {
		if (d.height > 0) {
			//insert below
			newParent = next;
			newIndex = 0;
			for (Feature child : next.getChildren()) {
				Dimension cd = FeatureUIHelper.getSourceLocation(child).getDifference(nextLocation);
				if (d.width / (double) d.height <= cd.width / (double) cd.height)
					break;
				else
					newIndex++;
			}
		}
		else {
			//insert left or right
			if (next.isRoot()) {
				//do not accept because root has no parent
				return false;
			}
			else {
				newParent = next.getParent();
				if (d.width < 0)
					newIndex = newParent.getChildIndex(next);
				else
					newIndex = newParent.getChildIndex(next) + 1;
			}
		}
		
		if (newParent == oldParent && oldParent.getChildIndex(feature) < newIndex)
			newIndex--;
		
		return true;
	}

	public static Feature calculateNext(Feature feature, Point referencePoint) {
		if (feature == null)
			return null;
		Feature next = feature;
		double distance = FeatureUIHelper.getTargetLocation(next).getDistance(referencePoint);
		for (Feature child : feature.getChildren()) {
			Feature childsNext = calculateNext(child, referencePoint);
			double newDistance = FeatureUIHelper.getTargetLocation(childsNext).getDistance(referencePoint);
			if (newDistance > 0 && newDistance < distance) {
				next = childsNext;
				distance = newDistance;
			}
		}
		return next;
	}

	public Feature getFeature() {
		return feature;
	}

	public Feature getNewParent() {
		return newParent;
	}

}
