package featureide_ui.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import featureide_core.model.FeatureModel;
import featureide_core.model.editing.Comparison;
import featureide_core.model.editing.ModelComparator;
import featureide_core.model.editing.evaluation.Evaluation;
import featureide_ui.FeatureModelUIActivator;
import featureide_ui.editors.GrammarEditor;

public class FeatureModelEditView extends ViewPart {

	private static final String DEFAULT_MESSAGE = "Open a feature model.";

	private static final Image DEFAULT_IMAGE = PlatformUI.getWorkbench()
			.getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);

	private static final Image ERROR_IMAGE = PlatformUI.getWorkbench()
			.getSharedImages().getImage(ISharedImages.IMG_OBJS_ERROR_TSK);

	private static final String CALCULATING_MESSAGE = "Calculating...";

	/**
	 * time in seconds after the calculation is aborted
	 */
	private static final int TIMEOUT = 20;

	private static final String TIMEOUT_MESSAGE = "Time out after " + TIMEOUT
			+ " sec!";

	private TreeViewer viewer;

	private GrammarEditor grammarEditor;

	private Job job;

	private IPartListener editorListener = new IPartListener() {

		public void partOpened(IWorkbenchPart part) {
		}

		public void partDeactivated(IWorkbenchPart part) {
		}

		public void partClosed(IWorkbenchPart part) {
			if (part == grammarEditor)
				setGrammarEditor(null);
		}

		public void partBroughtToTop(IWorkbenchPart part) {
			if (part instanceof IEditorPart)
				setGrammarEditor(part);
		}

		public void partActivated(IWorkbenchPart part) {
		}

	};

	private PropertyChangeListener modelListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			refresh();
		}
	};

	private long start;

	class TreeObject implements IAdaptable {
		private String name;
		private Image image;
		private TreeParent parent;

		public TreeObject(String name) {
			this(name, null);
		}

		public TreeObject(String name, Image image) {
			this.name = name;
			this.image = image;
		}

		public String getName() {
			return name;
		}

		public Image getImage() {
			return image;
		}

		public void setParent(TreeParent parent) {
			this.parent = parent;
		}

		public TreeParent getParent() {
			return parent;
		}

		public String toString() {
			return getName();
		}

		@SuppressWarnings("unchecked")
		public Object getAdapter(Class key) {
			return null;
		}
	}

	class TreeParent extends TreeObject {
		private ArrayList<TreeObject> children = new ArrayList<TreeObject>();

		public TreeParent(String name) {
			super(name);
		}

		public TreeParent(String name, Image image) {
			super(name, image);
		}

		public void addChild(TreeObject child) {
			children.add(child);
			child.setParent(this);
		}

		public void setChild(TreeObject child) {
			removeChildren();
			children.add(child);
			child.setParent(this);
		}

		public void removeChild(TreeObject child) {
			children.remove(child);
			child.setParent(null);
		}

		public void removeChildren() {
			for (TreeObject child : children)
				child.setParent(null);
			children.clear();
		}

		public TreeObject[] getChildren() {
			return (TreeObject[]) children.toArray(new TreeObject[children
					.size()]);
		}

		public boolean hasChildren() {
			return children.size() > 0;
		}
	}

	private TreeParent invisibleRoot = new TreeParent("");

	class ViewContentProvider implements IStructuredContentProvider,
			ITreeContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (parent.equals(getViewSite())) {
				if (invisibleRoot.getChildren().length == 0)
					return new TreeObject[] { new TreeObject(DEFAULT_MESSAGE,
							DEFAULT_IMAGE) };
				return getChildren(invisibleRoot);
			}
			return getChildren(parent);
		}

		public Object getParent(Object child) {
			if (child instanceof TreeObject) {
				return ((TreeObject) child).getParent();
			}
			return null;
		}

		public Object[] getChildren(Object parent) {
			if (parent instanceof TreeParent) {
				return ((TreeParent) parent).getChildren();
			}
			return new Object[0];
		}

		public boolean hasChildren(Object parent) {
			if (parent instanceof TreeParent)
				return ((TreeParent) parent).hasChildren();
			return false;
		}

	}

	class ViewLabelProvider extends LabelProvider {

		public String getText(Object obj) {
			return obj.toString();
		}

		public Image getImage(Object obj) {
			return ((TreeObject) obj).getImage();
		}

	}

	public FeatureModelEditView() {
		super();
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setInput(getViewSite());

		getSite().getPage().addPartListener(editorListener);
		IWorkbenchPage page = getSite().getPage();
		setGrammarEditor(page.getActiveEditor());
	}

	@Override
	public void dispose() {
		if (job != null) {
			if (job.getState() == Job.RUNNING)
				job.cancel();
			job = null;
		}
		getSite().getPage().removePartListener(editorListener);
		if (grammarEditor != null) {
			grammarEditor.getOriginalFeatureModel().removeListener(
					modelListener);
			grammarEditor.getFeatureModel().removeListener(modelListener);
			grammarEditor = null;
		}
		super.dispose();
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	private void setGrammarEditor(IWorkbenchPart activeEditor) {
		if (grammarEditor == activeEditor)
			return;

		if (grammarEditor != null) {
			grammarEditor.getOriginalFeatureModel().removeListener(
					modelListener);
			grammarEditor.getFeatureModel().removeListener(modelListener);
			grammarEditor = null;
		}

		if (activeEditor instanceof GrammarEditor) {
			grammarEditor = (GrammarEditor) activeEditor;
			grammarEditor.getOriginalFeatureModel().addListener(modelListener);
			grammarEditor.getFeatureModel().addListener(modelListener);
		}

		refresh();
	}

	private void refresh() {
		if (job != null && job.getState() == Job.RUNNING)
			job.cancel();

		invisibleRoot.setChild(new TreeObject(CALCULATING_MESSAGE,
				DEFAULT_IMAGE));
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if (!viewer.getControl().isDisposed())
					viewer.refresh();
			}
		});

		job = new Job("Updating Feature Model Edits") {
			@Override
			protected IStatus run(IProgressMonitor monitor) {
				calculateContent();
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						if (!viewer.getControl().isDisposed()) {
							viewer.refresh();
							viewer.expandAll();
						}
					}
				});
				return Status.OK_STATUS;
			}
		};
		job.setPriority(Job.LONG);
		job.schedule();

		final Job currentJob = job;
		new Thread(new Runnable() {
			public void run() {
				try {
					Thread.sleep(TIMEOUT * 1000);
				} catch (InterruptedException e) {
				}
				if (currentJob.getState() == Job.RUNNING) {
					currentJob.getThread().interrupt();
					if (job == currentJob) {
						invisibleRoot.setChild(new TreeObject(TIMEOUT_MESSAGE,
								ERROR_IMAGE));
						Display.getDefault().asyncExec(new Runnable() {
							public void run() {
								viewer.refresh();
							}
						});
					}
				}
			}
		}).start();
	}

	private void calculateContent() {
		if (grammarEditor == null) {
			invisibleRoot.removeChildren();
			return;
		}
		
		start = System.nanoTime();

		FeatureModel oldModel = grammarEditor.getOriginalFeatureModel();
		FeatureModel newModel = grammarEditor.getFeatureModel();

		if (oldModel.getRoot() == null || newModel.getRoot() == null)
			return;

		ModelComparator comparator = new ModelComparator(TIMEOUT);
		Comparison comparison = comparator.compare(oldModel, newModel);

		String message = null;
		Image image = null;
		if (comparison == Comparison.REFACTORING) {
			message = "Software Product Line has not changed: f \u21D4 f'";
			image = getImage("zero.gif");
		} else if (comparison == Comparison.GENERALIZATION) {
			message = "Software Product Line has got some new products: f \u21D2 f'";
			image = getImage("plus.gif");
		} else if (comparison == Comparison.SPECIALIZATION) {
			message = "Software Product Line has lost some products: f \u21D0 f'";
			image = getImage("minus.gif");
		} else if (comparison == Comparison.ARBITRARY) {
			message = "Software Product Line has lost some products and has got some new: f, f' incomparable";
			image = getImage("plusminus.gif");
		} else if (comparison == Comparison.OUTOFMEMORY) {
			message = "Out of memory error!";
			image = ERROR_IMAGE; 
		} else if (comparison == Comparison.TIMEOUT) {
			message = "SAT4J time out!";
			image = ERROR_IMAGE;
		} else {
			message = "An error has occured!";
			image = ERROR_IMAGE;
		}

		if (comparison == Comparison.GENERALIZATION && grammarEditor.getGrammarFile().getResource().getProject().getName().contains("EvaluationTest")) {
			Evaluation.evaluate();
		}

		message += " (" + (System.nanoTime() - start)/1000000 + "msec)";
		TreeParent root = new TreeParent(message, image);
		invisibleRoot.setChild(root);

		for (String output : comparator.getOutput())
			root.addChild(new TreeObject(output));
	}

	private Image getImage(String path) {
		return FeatureModelUIActivator.getImageDescriptor("icons/" + path).createImage();
	}

}